// -*- c++ -*-
// $Id: pen.h,v 1.9 2002/09/02 16:56:27 winkvist Exp $

#ifndef PEN_H
#define PEN_H

#include <qpixmap.h>
#include <qpainter.h>
#include "tool.h"

class Pen : public Tool
{
  Q_OBJECT
public:
  Pen(const QString & toolname);
  void mousePressEvent(QMouseEvent *e);
  void mouseMoveEvent(QMouseEvent *e);
  void mouseReleaseEvent(QMouseEvent *e);
  void activating();
  const char* pixmap() const;
signals:
  void modified();
private:
  int lastx;
  int lasty;
  bool drawing;
  int activeButton;
};

#endif // PEN_H

