<!DOCTYPE kpartgui>
<kpartgui version="2" name="kiconedit">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_new_window" append="new_merge"/>
  </Menu>
  <Menu name="edit"><text>&amp;Edit</text>
    <Action name="edit_paste_as_new" append="edit_paste_merge"/>
    <Action name="edit_clear" append="edit_paste_merge"/>
    <Saparator/>
    <Action name="edit_resize"/>
    <Action name="edit_grayscale"/>
  </Menu>
  <Menu name="tools"><text>&amp;Tools</text>
    <Action name="tool_find_pixel"/>
    <Action name="tool_freehand"/>
    <Action name="tool_rectangle"/>
    <Action name="tool_filled_rectangle"/>
    <Action name="tool_circle"/>
    <Action name="tool_filled_circle"/>
    <Action name="tool_ellipse"/>
    <Action name="tool_filled_ellipse"/>
    <Action name="tool_spray"/>
    <Action name="tool_flood_fill"/>
    <Action name="tool_line"/>
    <Action name="tool_eraser"/>
    <Action name="edit_select_rectangle"/>
    <Action name="edit_select_circle"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="options_show_grid" append="show_merge"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar" newline="true" fullwidth="true">
  <text>Main Toolbar</text>
  <Action name="view_zoom_in"/>
  <Action name="view_zoom_out"/>
  <Saparator/>
  <Action name="edit_resize"/>
  <Action name="edit_grayscale"/>
  <Saparator/>
  <Action name="options_show_grid"/>
  <Action name="help_whats_this"/>
</ToolBar>
<ToolBar name="toolsToolBar" position="left" noEdit="true">
  <text>Tools Toolbar</text>
  <Action name="tool_find_pixel"/>
  <Action name="tool_freehand"/>
  <Action name="tool_rectangle"/>
  <Action name="tool_filled_rectangle"/>
  <Action name="tool_circle"/>
  <Action name="tool_filled_circle"/>
  <Action name="tool_ellipse"/>
  <Action name="tool_filled_ellipse"/>
  <Action name="tool_spray"/>
  <Action name="tool_flood_fill"/>
  <Action name="tool_line"/>
  <Action name="tool_eraser"/>
  <Action name="edit_select_rectangle"/>
  <Action name="edit_select_circle"/>
</ToolBar>
<ToolBar name="paletteToolBar" position="right" noEdit="true">
  <text>Pallette Toolbar</text>
</ToolBar>
</kpartgui>

