<!DOCTYPE kpartgui>
<kpartgui name="KPovModelerPart" version="1">
<MenuBar>
   <Menu name="edit"><text>&amp;Edit</text>
      <Action name="edit_cut"/>
      <Action name="edit_copy"/>
      <Action name="edit_paste"/>
      <Action name="edit_delete"/>
   </Menu>
</MenuBar>

<ToolBar name="mainToolBar">
  <Action name="edit_undo"/>
  <Action name="edit_redo"/>
  <Separator/>
  <Action name="edit_cut"/>
  <Action name="edit_copy"/>
  <Action name="edit_paste"/>
  <Action name="edit_delete"/>
</ToolBar>
</kpartgui>
