<!DOCTYPE kpartgui>
<kpartgui name="KGhostViewShell" version="5">
<State name="initState">
    <Disable>
        <Action name="file_print" />
	<Action name="view_redisplay" />
	<Action name="fit_to_page" />
    </Disable>
</State>
<State name="documentState">
    <Enable>
        <Action name="file_print" />
	<Action name="view_redisplay" />
	<Action name="fit_to_page" />
    </Enable>
</State>
<MenuBar>
  <Menu name="file">
    <DefineGroup name="file_save" append="save_merge" />
    <DefineGroup name="file_print" append="print_merge" />
  </Menu>
  <Menu name="view">
    <Action name="fit_to_page"/>
  </Menu>
  <Merge/>
</MenuBar>
</kpartgui>

