<!DOCTYPE kpartgui>
<kpartgui name="kdvi_part" version="1">
<MenuBar>
 <Menu name="file"><text>File</text>
  <Action name="info_dvi" group="file_print" append="options_show_merge"/>
  <Menu name="export" group="file_save" ><text>Export As</text>
   <Action name="export_postscript" append="options_show_merge"/>
   <Action name="export_pdf" append="options_show_merge"/>
   <Action name="export_text" append="options_show_merge"/>
  </Menu>
 </Menu>

 <Menu name="go_document"><text>Go</text>
  <Action name="go_back"/>
  <Action name="go_forward"/>
 </Menu>

 <Menu name="edit"><text>Edit</text>
  <Action name="copy_text"/>
  <Action name="edit_select_all"/>
  <Separator/>
  <Action name="find"/>
  <Action name="findnext"/>
  <Action name="findprev"/>
 </Menu>

 <Menu name="settings"><text>Settings</text>
  <Action name="enable_msgs"/>
  <Action name="settings_dvi" append="options_show_merge"/>
 </Menu>

 <Menu name="help"><text>Help</text>
  <Action name="help_tipofday"/>
  <Separator/>
  <Action name="help_dvi" append="options_show_merge"/>
  <Action name="about_kdvi" append="options_show_merge"/>
  <Action name="bug_dvi" append="options_show_merge"/>
 </Menu>
</MenuBar>

<ToolBar name="mainToolBar">
  <Action name="go_back"/>
  <Action name="go_forward"/>
</ToolBar>

</kpartgui>
