// -*- c++ -*-
// $Id: circle.h,v 1.9 2002/09/02 16:56:27 winkvist Exp $

#ifndef CIRCLE_H
#define CIRCLE_H

#include "tool.h"

class Circle : public Tool
{
  Q_OBJECT
public:
  Circle();
  Circle(const QString & toolname);
  void mousePressEvent(QMouseEvent *e);
  void mouseMoveEvent(QMouseEvent *e);
  void mouseReleaseEvent(QMouseEvent *e);
  void activating();
  const char* pixmap() const;
signals:
  void modified();

private:
  int startx, starty;
  int lastx, lasty;
  bool drawing;
  int activeButton;
};

#endif // CIRCLE_H

