<!DOCTYPE kpartgui>
<kpartgui version="1" name="kfax">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_add_fax" append="open_merge"/>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Action name="view_rotate"/>
    <Action name="view_mirror"/>
    <Action name="view_flip"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="go_first"/>
  <Action name="go_last"/>
  <Separator/>
  <Action name="view_zoom_in"/>
  <Action name="view_zoom_out"/>
</Toolbar>
</Kpartgui>
