// $Id: version.h,v 1.14 2002/02/08 21:30:40 rich Exp $

#define APPVERSTR "0.6.2" 
#define APPNAME "kpaint"
#define APPHELPCONTENTS "kpaint.html"
#define APPMAJVER 6
#define APPMINVER 2
#define APPAUTHOR1 "Richard J. Moore"
#define APPAUTHOR1MAIL "rich@kde.org"
#define APPAUTHOR2 "Jörg Habenicht"
#define APPAUTHOR2MAIL "j.habenicht@europemail.com"
#define APPAUTHOR APPAUTHOR1 " <" APPAUTHOR1MAIL ">\n" APPAUTHOR2 " <" APPAUTHOR2MAIL ">\n"
#define APPLICENSE "  KPaint is released under the  \n" \
                   "  GNU Public License agreement,  \n" \
                   "  see the online documentation  \n" \
                   "            for details."

