/***************************************************************************
                          formathelp.h  -  description                              
                             -------------------                                         
    begin                : Mon Dec 27 1999                                           
    copyright            : (C) 1999 by Klaas Freitag                         
    email                : freitag@suse.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#define HELP_BMP i18n("The <big>bitmap-format</big> is a well known format,\n" \
        "often used for 256 color images under "  \
        "MS Windows.\n Suitable for color and " \
        "<bold>lineart-pictures</bold>\n" )
#define HELP_PNM i18n("Portable Anymap\n" \
        ""\
        "" )
#define HELP_JPG i18n("Jpeg is a high compression,\nquality " \
        "losing format for color\npictures " \
        "with many different colors." \
        "" )

#define HELP_EPS i18n("EPS is Encapsulated Postscript.\n " \
        "Initially it's a printer definition\n " \
        "language. Use this format if you\n" \
        "want to print the image or use\n" \
        "it with e.g. TeX" )
