<!DOCTYPE kpartgui>
<kpartgui name="KViewViewer Part" version="4">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<Action name="file_save" group="save_merge_group"/>
			<Action name="file_save_as" group="save_merge_group"/>
		</Menu>
		<Menu name="view"><text>&amp;View</text>
			<Action name="zoomin"/>
			<Action name="zoomout"/>
			<Action name="view_zoom"/>
			<Separator/>
			<Action name="fittowin"/>
			<Separator/>
			<Action name="flip"/>
			<Action name="rotate"/>
		</Menu>
		<Menu name="settings"><text>&amp;Settings</text>
			<Action name="show_scrollbars" group="show_merge_group" />
		</Menu>
	</MenuBar>
	<ToolBar name="mainToolBar"><text>Main Toolbar</text>
		<Action name="file_save_as"/>
		<Action name="zoomin"/>
		<Action name="zoomout"/>
		<Action name="view_zoom"/>
		<Action name="rotate"/>
	</ToolBar>
	<Menu name="popupmenu">
		<Separator/>
		<Action name="saveimageas" group="file_group"/>
	</Menu>
</kpartgui>
