<!DOCTYPE kpartgui>
<kpartplugin name="kviewscanner" library="kview_scannerplugin">
	<MenuBar>
		<Menu name="tools"><Text>&amp;Tools</Text>
			<Action name="plugin_scan"/>
		</Menu>
	</MenuBar>
	<ToolBar name="extraToolBar">
        <text>&amp;Extra Toolbar</text>
		<Action name="plugin_scan"/>
	</ToolBar>
</kpartplugin>
