#ifndef __KVIEWSHELL_H__
#define __KVIEWSHELL_H__


#include <qstring.h>
#include <qevent.h>


#include <kparts/mainwindow.h>
#include <kurl.h>


class KRecentFilesAction;
class ScrollBox;
class KToggleAction;
class KViewPart_Iface;
class KHelpMenu;
class KAboutData;


class QLabel;

class KViewShell : public KParts::MainWindow
{
  Q_OBJECT

public:
  KStatusBar *statusbar;
  KStatusBar *action;
  
  KViewShell(QString partname, KAboutData *);
  virtual ~KViewShell();

public slots:
  void openURL(const KURL &url);


protected slots:
  void slotFileOpen();
  void slotShowToolBar();
  void slotShowStatusBar();
  void slotQuit();
  void slotEditToolbar();
  void slotFileClose();

  void slotChangeZoomText(const QString &);
  void slotChangePageText(const QString &);
  void slotChangeSizeText(const QString &);

protected:
  void readSettings();
  void writeSettings();

  void checkActions();

  void dragEnterEvent(QDragEnterEvent *event);
  void dropEvent(QDropEvent *event);


private:
  QString _partname;
  KViewPart_Iface *view;

  KRecentFilesAction *recent;
  QString cwd;

  KAction *closeAction, *reloadAction;
  KToggleAction *showtoolbar, *showstatusbar;

  KHelpMenu *m_helpMenu;
};


#endif
