<!DOCTYPE kpartgui>
<kpartgui name="KViewShell" version="1">
<MenuBar>
 <Menu name="file">
   <Action name="reload" append="open_merge" />
   <DefineGroup name="file_print" append="print_merge" />
   <DefineGroup name="file_save" append="save_merge" />
 </Menu>
 <Merge/>
</MenuBar>
<StatusBar/>
</kpartgui>

