/*
**************************************************************************
                                 description
                             --------------------
    copyright            : (C) 2000-2001 by Andreas Zehender
    email                : zehender@kde.org
**************************************************************************

**************************************************************************
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
**************************************************************************/


#include "pminserterrors.h"
#include <klocale.h>

PMInsertErrors::PMInsertErrors( )
      : Base( )
{
}

PMInsertErrors::~PMInsertErrors( )
{
}

QString PMInsertErrors::description( ) const
{
   return i18n( "insert errors" );
}

bool PMInsertErrors::isA( PMObjectType t ) const
{
   if( t == PMTInsertErrors )
      return true;
   return Base::isA( t );
}
