/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

void ImageListDialog::init()
{
    kdDebug( 4630 ) << k_funcinfo << endl;
    m_pInterval->setRange( 0, 60000 );
    noSort();
}

void ImageListDialog::loadList()
{
    kdDebug( 4630 ) << k_funcinfo << endl;
/*
    KURL url = KFileDialog::getOpenURL( ":load_list", QString::null, this );
    if( url.isEmpty() )
	return;
    
    QString tempfile;
    if( ! KIO::NetAccess::download( url, tempfile ) )
    {
	KMessageBox::error( this, i18n( "Could not load\n%1" ).arg( url.prettyURL() ) );
	return;
    }
    QFile file( tempfile );
    if( file.open( IO_ReadOnly ) )
    {
	QTextStream t( &file );
	if( t.readLine() == "[KView Image List]" )
	{
	    //clear old image list
	    m_pListView->clear()
	    QStringList list;
	    while( ! t.eof() )
		list << t.readLine();
	    addURLList( list );
	}
	else
	{
	    KMessageBox::error( this, i18n( "Wrong format\n%1" ).arg( url.prettyURL() ) );
	}
	file.close();
    }
    KIO::NetAccess::removeTempFile( tempfile );*/
}

void ImageListDialog::saveList()
{
    kdDebug( 4630 ) << k_funcinfo << endl;
}


void ImageListDialog::noSort()
{
    kdDebug( 4630 ) << k_funcinfo << endl;
    m_pListView->setSorting( 1000 );
}


