//-*-C++-*-
/*
**************************************************************************
                                 description
                             --------------------
    copyright            : (C) 2002 by Andreas Zehender
    email                : zehender@kde.org
**************************************************************************

**************************************************************************
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
**************************************************************************/

#ifndef PMOBJECTTYPE_H
#define PMOBJECTTYPE_H

/** Enum for real time type information. */
enum PMObjectType { PMTNone, PMTObject, PMTCompositeObject, PMTNamedObject,
                    PMTGraphicalObject, PMTSolidObject, PMTGlobalSettings,
                    PMTScene, PMTInsertErrors, PMTBox, PMTSphere,
                    PMTCylinder, PMTCone, PMTTorus, PMTCSG,
                    PMTBlob, PMTBlobCylinder, PMTBlobSphere,
                    PMTPlane, PMTDisc, PMTBicubicPatch, PMTMesh,
                    PMTPolygon, PMTTriangle, PMTLathe, PMTPrism,
                    PMTSurfaceOfRevolution, PMTSuperquadricEllipsoid,
                    PMTPolynom, PMTHeightField, PMTJuliaFractal, PMTText,
                    PMTComment, PMTRaw,
                    PMTTranslate, PMTScale, PMTRotate, PMTMatrix,
                    PMTCamera, PMTLight, PMTLooksLike,
                    PMTBoundedBy, PMTClippedBy,
                    PMTTextureBase,
                    PMTTexture, PMTPigment, PMTSolidColor,
                    PMTListPattern, PMTTextureList, PMTColorList,
                    PMTPigmentList, PMTNormalList, PMTDensityList,
                    PMTDeclare, PMTObjectLink, PMTQuickColor,
                    PMTPattern, PMTBlendMapModifiers, PMTImageMap,
                    PMTTextureMapBase, PMTTextureMap, PMTColorMap, 
                    PMTPigmentMap, PMTNormalMap, PMTSlopeMap, PMTDensityMap,
                    PMTBumpMap, PMTMaterialMap, PMTWarp, PMTNormal, PMTSlope, 
                    PMTFinish, PMTSkySphere, PMTBackground,
                    PMTRainbow, PMTFog, PMTInterior, 
                    PMTMedia, PMTMaterial, PMTDensity };

#endif
