/** 
 * Copyright (C) 1997-2002 the KGhostView authors. See file AUTHORS.
 * 	
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MESSAGES_H
#define MESSAGES_H

#include <qdialog.h>
#include <qframe.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qkeycode.h>
#include <qcombobox.h>

class MessagesDialog : public QDialog {

	Q_OBJECT

public:
	MessagesDialog( QWidget *parent, const char *name );
	QMultiLineEdit *messageBox;
	QPushButton* cancel;
	QFrame *frame;
	
public slots:
	void clear();
	
protected slots:
	void resizeEvent( QResizeEvent * );
};

#endif
